@@ -1,5 +1,6 @@
 import collections
 import typing
+from enum import Enum
 
 import attr
 
@@ -9,20 +10,22 @@ from ._error import (
     BadTypeError,
     CallableError,
     EmptyError,
+    LiteralError,
     TupleError,
     UnionError,
+    UnsupportedLiteralError,
 )
 
 try:
-    from collections.abc import Mapping, MutableMapping, Callable
+    from collections.abc import Callable, Mapping, MutableMapping
 except ImportError:
-    from collections import Mapping, MutableMapping, Callable
+    from collections import Callable, Mapping, MutableMapping
 
 try:
-    from inspect import signature
+    from inspect import Parameter, Signature, signature
 except ImportError:
     # silencing type error so mypy doesn't complain about duplicate import
-    from funcsigs import signature  # type: ignore
+    from funcsigs import Parameter, Signature, signature  # type: ignore
 
 try:
     from itertools import zip_longest
@@ -31,10 +34,17 @@ except ImportError:
     from itertools import izip_longest as zip_longest  # type: ignore
 
 try:
-    from typing import ForwardRef  # type: ignore # Not in stubs
+    from typing import ForwardRef
 except ImportError:
     from typing import _ForwardRef as ForwardRef  # type: ignore # Not in stubs
 
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal  # type: ignore
+
+SupportedLiterals = (int, str, bool, Enum)
+
 
 class _StringAnnotationError(Exception):
     """Raised when we find string annotations in a class."""
@@ -113,14 +123,12 @@ def _validate_elements(attribute, value, expected_type):
         # These base_types happen when you have string annotations and cannot
         # be used in isinstance.
         raise _StringAnnotationError()
-
-    if base_type != typing.Union and not isinstance(  # type: ignore
-        value, base_type
-    ):
-        raise AttributeTypeError(value, attribute)
-
-    if base_type == typing.Union:  # type: ignore
+    elif base_type == Literal or base_type == type(Literal):  # type: ignore
+        _handle_literal(attribute, value, expected_type)
+    elif base_type == typing.Union:  # type: ignore
         _handle_union(attribute, value, expected_type)
+    elif not isinstance(value, base_type):
+        raise AttributeTypeError(value, attribute)
     elif base_type in SimilarTypes.List:
         _handle_set_or_list(attribute, value, expected_type)
     elif base_type in SimilarTypes.Dict:
@@ -136,6 +144,8 @@ def _get_base_type(type_):
         base_type = type_.__origin__  # type: typing.Type[typing.Any]
     elif is_newtype(type_):
         base_type = type_.__supertype__
+    elif getattr(type_, "__args__", None) or getattr(type_, "__values__", None):
+        base_type = type(type_)
     else:
         base_type = type_
 
@@ -145,6 +155,7 @@ def _get_base_type(type_):
 def _type_matching(actual, expected):
     actual = actual.__supertype__ if is_newtype(actual) else actual
     expected = expected.__supertype__ if is_newtype(expected) else expected
+    actual = type(None) if actual is None else actual
 
     if expected == actual or expected == typing.Any:
         return True
@@ -173,23 +184,66 @@ def _type_matching(actual, expected):
     return False
 
 
+def _handle_callable_arg(
+    attribute, _signature, expected_type, actual, expected
+):
+    if not _type_matching(actual, expected):
+        raise CallableError(
+            attribute, _signature, expected_type, actual, expected
+        )
+
+
 def _handle_callable(attribute, callable_, expected_type):
     _signature = signature(callable_)
-    callable_args = [
-        param.annotation for param in _signature.parameters.values()
-    ]
-    callable_args.append(_signature.return_annotation)
-    if not expected_type.__args__:  # type: ignore
+    empty = Signature.empty
+    callable_args = list(_signature.parameters.values())
+    callable_return = _signature.return_annotation
+    if not getattr(expected_type, "__args__", None):
         return  # No annotations specified on type, matches all Callables
 
-    for callable_arg, expected_arg in zip_longest(
-        callable_args, expected_type.__args__  # type: ignore
-    ):
-        if not _type_matching(callable_arg, expected_arg):
+    expected_args = expected_type.__args__[:-1]  # type: ignore
+    expected_return = expected_type.__args__[-1]  # type: ignore
+    for callable_arg, expected_arg in zip_longest(callable_args, expected_args):
+        callable_type = (
+            empty if callable_arg is None else callable_arg.annotation
+        )
+        callable_default = (
+            empty if callable_arg is None else callable_arg.default
+        )
+        callable_kind = empty if callable_arg is None else callable_arg.kind
+
+        if expected_arg is None and callable_default is not empty:
+            # The callable accepts more arguments than expected, but still
+            # matches the expected signature because these arguments are
+            # optional (have a default value)
+            continue
+
+        if callable_kind == Parameter.KEYWORD_ONLY:
+            # The callable has a keyword-only parameter and either
+            # 1) expected_arg was not None, which means that the Callable type
+            #    hint tries to pass an argument to it (but Callable only works
+            #    with positional arguments)
+            # or
+            # 2) expected_arg is None and callable_default is empty, which
+            #    means that this keyword-only parameter doesn't have a default
+            #    value (so you can't call this function by passing only
+            #    positional arguments)
             raise CallableError(
-                attribute, _signature, expected_type, callable_arg, expected_arg
+                attribute,
+                _signature,
+                expected_type,
+                callable_kind,
+                expected_arg,
             )
 
+        _handle_callable_arg(
+            attribute, _signature, expected_type, callable_type, expected_arg
+        )
+
+    _handle_callable_arg(
+        attribute, _signature, expected_type, callable_return, expected_return
+    )
+
 
 def _handle_set_or_list(attribute, container, expected_type):
     (element_type,) = expected_type.__args__  # type: ignore
@@ -248,6 +302,38 @@ def _handle_union(attribute, value, expected_type):
     raise UnionError(value, attribute.name, expected_type)
 
 
+def _handle_literal(attribute, value, expected_type):
+    flattened_literals = _flatten_literals(expected_type)
+
+    if not any(
+        value == literal and type(value) == type(literal)
+        for literal in flattened_literals
+    ):
+        raise LiteralError(attribute.name, value, flattened_literals)
+
+
+def _flatten_literals(literals):  # type: ignore
+    meta = "__args__" if hasattr(literals, "__args__") else "__values__"
+    extracted_literals = getattr(literals, meta, None)
+
+    flattened_literals = []
+    unsupported_literals = []
+
+    for literal in extracted_literals:
+        base_type = _get_base_type(literal)
+        if base_type == Literal or base_type == type(Literal):  # type: ignore
+            flattened_literals.extend(_flatten_literals(literal))
+        elif literal is None or isinstance(literal, SupportedLiterals):
+            flattened_literals.append(literal)
+        else:
+            unsupported_literals.append(literal)
+
+    if unsupported_literals:
+        raise UnsupportedLiteralError(unsupported_literals)
+
+    return flattened_literals
+
+
 # -----------------------------------------------------------------------------
 # Copyright 2019 Bloomberg Finance L.P.
 #
